@@ -1,6 +1,6 @@
 ;;; ada-imenu.el - Ada mode interface to imenu for Ada Mode  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2012, 2013, 2015  Free Software Foundation, Inc.
+;; Copyright (C) 2012, 2013, 2015, 2019  Free Software Foundation, Inc.
 ;;
 ;; Author: Simon Wright <simon@pushface.org>
 ;; Contributors: see ada-mode.el, and specifically Christian Egli
@@ -19,12 +19,12 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 ;;
 ;;; History: see ada_mode.el
 ;;
 
-(require 'ada-mode)
+(require 'ada-core)
 (require 'imenu)
 
 (defconst ada--imenu-comment-re "\\([ \t]*--.*\\)?")
