@@ -1,14 +1,14 @@
 ;;; ada-mode.el --- major-mode for editing Ada sources  -*- lexical-binding:t -*-
 ;;
-;; Copyright (C) 1994, 1995, 1997 - 2022  Free Software Foundation, Inc.
+;; Copyright (C) 1994, 1995, 1997 - 2023  Free Software Foundation, Inc.
 ;;
 ;; Author: Stephen Leake <stephen_leake@stephe-leake.org>
 ;; Maintainer: Stephen Leake <stephen_leake@stephe-leake.org>
 ;; Keywords: languages
 ;;  ada
-;; Version: 8.0.3
-;; package-requires: ((uniquify-files "1.0.1") (wisi "4.1.1") (gnat-compiler "1.0.0") (emacs "25.3"))
-;; url: http://www.nongnu.org/ada-mode/
+;; Version: 8.0.5
+;; package-requires: ((uniquify-files "1.0.4") (wisi "4.2.2") (gnat-compiler "1.0.2") (emacs "25.3"))
+;; url: https://www.nongnu.org/ada-mode/
 ;;
 ;; This file is part of GNU Emacs.
 ;;
@@ -23,7 +23,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 ;;
 ;;; Usage:
 ;;
@@ -117,7 +117,7 @@
 (defun ada-mode-version ()
   "Return Ada mode version."
   (interactive)
-  (let ((version-string "8.0.2"))
+  (let ((version-string "8.0.5"))
     (if (called-interactively-p 'interactive)
 	(message version-string)
       version-string)))
@@ -156,7 +156,7 @@ nil, only the file name."
   :type 'boolean
   :safe #'booleanp)
 
-;;;; keymap and menus
+;;;; keymaps and menus
 
 (defvar ada-mode-map
   (let ((map (make-sparse-keymap)))
@@ -164,27 +164,18 @@ nil, only the file name."
 
     ;; global-map has C-x ` 'next-error
     (define-key map [return] 	 'wisi-case-adjust-interactive)
-    (define-key map "\C-c`" 	 'ada-show-secondary-error)
+    (define-key map "\C-c`" 	 'gnat-show-secondary-error)
     (define-key map "\C-c;"      (lambda () (error "use M-; instead"))) ; comment-dwim
-    (define-key map "\C-c<" 	 'ada-goto-declaration-start)
-    (define-key map "\C-c>" 	 'ada-goto-declaration-end)
     (define-key map "\C-c\M-`" 	 'wisi-fix-compiler-error)
     (define-key map "\C-c\C-a" 	 'ada-align)
     (define-key map "\C-c\C-b" 	 'ada-make-subprogram-body)
     (define-key map "\C-c\C-c"   'ada-build-make)
-    (define-key map "\C-c\C-d" 	 'wisi-goto-spec/body)
-    (define-key map "\C-c\M-d" 	 'wisi-show-declaration-parents)
     (define-key map "\C-c\C-e" 	 'wisi-skel-expand)
     (define-key map "\C-c\C-f" 	 'wisi-show-parse-error)
-    (define-key map "\C-c\C-i" 	 'wisi-indent-statement)
-    (define-key map [3 backtab] 'wisi-indent-containing-statement);; C-c backtab, translated from C-c S-tab
     (define-key map "\C-\M-i"    'completion-at-point)
-    (define-key map "\C-c\C-l" 	 'wisi-show-local-references)
     (define-key map "\C-c\C-m"   'ada-build-set-make)
-    (define-key map "\C-c\C-n" 	 'forward-sexp)
     (define-key map "\C-c\M-n" 	 'wisi-skel-next-placeholder)
     (define-key map "\C-c\C-o" 	 'ada-find-other-file)
-    (define-key map "\C-c\C-p" 	 'backward-sexp)
     (define-key map "\C-c\M-p" 	 'wisi-skel-prev-placeholder)
     (define-key map "\C-c\C-q" 	 'wisi-refresh-prj-cache)
     (define-key map "\C-c\C-r" 	 'wisi-show-references)
@@ -193,8 +184,6 @@ nil, only the file name."
     (define-key map "\C-c\C-t" 	 'ada-find-file)
     (define-key map "\C-c\C-v"   'ada-build-check)
     (define-key map "\C-c\C-w" 	 'wisi-case-adjust-at-point)
-    (define-key map "\C-c\C-x"   'wisi-show-overriding)
-    (define-key map "\C-c\M-x"   'wisi-show-overridden)
     (define-key map "\C-c\C-y" 	 'wisi-case-create-exception)
     (define-key map "\C-c\C-\M-y" 'wisi-case-create-partial-exception)
     (define-key map [C-down-mouse-3] 'ada-popup-menu)
@@ -225,32 +214,21 @@ nil, only the file name."
      ["Clear current project"         wisi-prj-clear-current 	       t]
     )
     ("Build"
-     ["Next compilation error"     next-error 		    t]
-     ["Show secondary error"       ada-show-secondary-error t]
-     ["Fix compilation error"      wisi-fix-compiler-error  t]
-     ["Show last parse error"      wisi-show-parse-error    t]
-     ["Check syntax"               ada-build-check 	    t]
-     ["Show main"                  ada-build-show-main 	    t]
-     ["Build"                      ada-build-make 	    t]
-     ["Set main and Build"         ada-build-set-make 	    t]
-     ["Run"                        ada-build-run 	    t]
+     ["Next compilation error/xref"  next-error 	      t]
+     ["Show secondary error"         gnat-show-secondary-error t]
+     ["Fix compilation error"        wisi-fix-compiler-error  t]
+     ["Show last parse error"        wisi-show-parse-error    t]
+     ["Check syntax"                 ada-build-check 	      t]
+     ["Show main"                    ada-build-show-main      t]
+     ["Build"                        ada-build-make 	      t]
+     ["Set main and Build"           ada-build-set-make       t]
+     ["Run"                          ada-build-run 	      t]
      )
     ("Navigate"
-     ["Other file"                    ada-find-other-file          t]
-     ["Find file in project"          project-find-file            t]
-     ["Goto declaration/body"         wisi-goto-spec/body          t]
-     ["Goto next statement keyword"   forward-sexp 			     wisi-parser-shared]
-     ["Goto prev statement keyword"   backward-sexp 			     wisi-parser-shared]
-     ["Goto subprogram/package start" ada-goto-declaration-start 	     wisi-parser-shared]
-     ["Goto subprogram/package end"   ada-goto-declaration-end 		     wisi-parser-shared]
-     ["Goto declarative region start" ada-goto-declarative-region-start      wisi-parser-shared]
-     ["Goto containing statement start" wisi-goto-containing-statement-start wisi-parser-shared]
-     ["Show parent declarations"        wisi-show-declaration-parents 	     (eq ada-xref-backend 'gpr_query)]
-     ["Show all references (classwide)" xref-find-references                 t]
-     ["Show all direct references"      wisi-show-references                 t]
-     ["Show local references"         wisi-show-local-references (eq ada-xref-backend 'gpr_query)]
-     ["Show overriding"               wisi-show-overriding       (eq ada-xref-backend 'gpr_query)]
-     ["Show overridden"               wisi-show-overridden       (eq ada-xref-backend 'gpr_query)]
+     ["Other file"                    ada-find-other-file        t]
+     ["Find file in project"          project-find-file          t]
+     ["Show all references (classwide)" xref-find-references     t]
+     ["Show all direct references"      wisi-show-references     t]
      ["Goto prev position current buffer" pop-to-mark-command 	 t]
      ["Goto prev position other buffer"   pop-global-mark 	 t]
      ["Next placeholder"              wisi-skel-next-placeholder t]
@@ -290,6 +268,50 @@ nil, only the file name."
      ["Restart parser"                wisi-kill-parser            wisi-parser-shared]
      )))
 
+(defvar ada--statement-minor-mode-map
+  (let ((map (make-sparse-keymap)))
+    (define-key map "\C-c<" 	 'ada-goto-declaration-start)
+    (define-key map "\C-c>" 	 'ada-goto-declaration-end)
+    (define-key map "\C-c\C-i" 	 'wisi-indent-statement)
+    (define-key map [3 backtab] 'wisi-indent-containing-statement);; C-c backtab, translated from C-c S-tab
+    (define-key map "\C-c\C-n" 	 'forward-sexp)
+    (define-key map "\C-c\C-p" 	 'backward-sexp)
+
+    map)
+  "Local keymap used for Ada statement motion minor mode.")
+
+(defvar ada--statement-minor-mode-menu (make-sparse-keymap "Ada statement motion"))
+(easy-menu-define ada--statement-minor-mode-menu ada--statement-minor-mode-map
+  "Menu keymap for Ada statement motion minor mode"
+  '("Ada statement motion"
+    ["Goto next statement keyword"   forward-sexp 			     wisi-parser-shared]
+    ["Goto prev statement keyword"   backward-sexp 			     wisi-parser-shared]
+    ["Goto subprogram/package start" ada-goto-declaration-start 	     wisi-parser-shared]
+    ["Goto subprogram/package end"   ada-goto-declaration-end 		     wisi-parser-shared]
+    ["Goto declarative region start" ada-goto-declarative-region-start      wisi-parser-shared]
+    ["Goto containing statement start" wisi-goto-containing-statement-start wisi-parser-shared]
+    ))
+
+(defvar ada--gpr-query-minor-mode-map
+  (let ((map (make-sparse-keymap)))
+    (define-key map "\C-c\C-d" 	 'wisi-goto-spec/body) ;; uses xref backend
+    (define-key map "\C-c\M-d" 	 'wisi-show-declaration-parents)
+    (define-key map "\C-c\C-l" 	 'wisi-show-local-references)
+    (define-key map "\C-c\C-x"   'wisi-show-overriding)
+    (define-key map "\C-c\M-x"   'wisi-show-overridden)
+    map)
+  "Local keymap used for gpr query minor mode.")
+
+(defvar ada--gpr-query-minor-mode-menu (make-sparse-keymap "gpr-query"))
+(easy-menu-define gpr-query-menu ada--gpr-query-minor-mode-map "Menu keymap for gpr-query minor mode"
+  '("gpr-query"
+    ["Goto declaration/body"    wisi-goto-spec/body           t]
+    ["Show parent declarations" wisi-show-declaration-parents t]
+    ["Show local references"    wisi-show-local-references    t]
+    ["Show overriding"          wisi-show-overriding          t]
+    ["Show overridden"          wisi-show-overridden          t]
+    ))
+
 (easy-menu-define ada-context-menu nil
   "Context menu keymap for Ada mode"
   '("Ada"
@@ -691,10 +713,13 @@ Also sets `ff-function-name' for `ff-pre-load-hook'."
 	 (error ""))))
 
     (eglot
-     (when (eglot--server-capable :DocumentSymbol)
-	 ;; eglot/LSP maybe supports "which function" via DocumentSymbol, which
-	 ;; ada_language_server version 22 does not support.
-	 (display-warning 'ada "ada-mode which-function via eglot not yet implemented."))
+     (if (eglot--server-capable :DocumentSymbol)
+	(progn
+	  ;; eglot/LSP maybe supports "which function" via DocumentSymbol, which
+	  ;; ada_language_server version 22 does not support.
+	  (display-warning 'ada "ada-mode which-function via eglot not yet implemented.")
+	  "")
+       "")
      )
 
     (none "")
@@ -715,7 +740,9 @@ Also sets `ff-function-name' for `ff-pre-load-hook'."
   (interactive)
   (and wisi-parser-shared
        (save-excursion
-	 (memq (wisi-cache-nonterm (wisi-goto-statement-start)) '(use_clause with_clause))
+	 (let ((start-cache (wisi-goto-statement-start)))
+	   (when start-cache
+	     (memq (wisi-cache-nonterm (wisi-goto-statement-start)) '(use_clause with_clause))))
 	 )))
 
 (defun ada-goto-subunit-name ()
@@ -852,53 +879,6 @@ the file name."
     (speedbar-add-supported-extension body))
   )
 
-(defun ada-show-secondary-error ()
-  "Show the next secondary file reference in the compilation buffer.
-A secondary file reference is defined by text having text
-property `ada-secondary-error'.  These can be set by
-compiler-specific compilation filters."
-  (interactive)
-
-  ;; preserving the current window works only if the frame
-  ;; doesn't change, at least on Windows.
-  (let ((start-buffer (current-buffer))
-	pos item file)
-    (when (eq major-mode 'compilation-mode)
-      (setq next-error-last-buffer (current-buffer)))
-    ;; We use `pop-to-buffer', not `set-buffer', so point is correct
-    ;; for the current window showing next-error-last-buffer, and
-    ;; moving point in that window works. But that might eat an
-    ;; `other-frame-window-mode' prefix, which the user means to apply
-    ;; to ’ada-goto-source’ below; disable that temporarily.
-    (let ((display-buffer-overriding-action nil))
-      (pop-to-buffer next-error-last-buffer nil t)
-      (setq pos (next-single-property-change (point) 'ada-secondary-error))
-      (unless pos
-	;; probably at end of compilation-buffer, in new compile
-	(goto-char (point-min))
-	(setq pos (next-single-property-change (point) 'ada-secondary-error)))
-
-      (when pos
-	(setq item (get-text-property pos 'ada-secondary-error))
-	;; file-relative-name handles absolute Windows paths from
-	;; g++. Do this in compilation buffer to get correct
-	;; default-directory.
-	(setq file (file-relative-name (nth 0 item)))
-
-	;; Set point in compilation buffer past this secondary error, so
-	;; user can easily go to the next one.
-	(goto-char (next-single-property-change (1+ pos) 'ada-secondary-error)))
-
-      (pop-to-buffer start-buffer nil t);; for windowing history
-      )
-    (when item
-      (wisi-goto-source
-       file
-       (nth 1 item); line
-       (nth 2 item); column
-       ))
-    ))
-
 (defun ada-goto-declaration-start-1 (include-type)
   "Subroutine of `ada-goto-declaration-start'."
   (let ((start (point))
@@ -1296,7 +1276,7 @@ Point must have been set by `ada-wisi-find-begin'."
 		    (buffer-substring-no-properties
 		     (point)
 		     (progn
-		       (skip-syntax-forward "ws._")
+		       (skip-syntax-forward "w-._")
 		       (point)))
 		    ";"))
       (if (search-forward-regexp end-regexp nil t)
@@ -1599,6 +1579,14 @@ Unless WAIT, does not wait for parser to respond. Returns the parser object."
     (wisi-parse-require-process parser :nowait (not wait))
     parser))
 
+(define-minor-mode ada--statement-minor-mode
+  "Minor mode enabling statement motion using the wisi parser."
+  :lighter nil :interactive nil)
+
+(define-minor-mode ada--gpr-query-minor-mode
+  "Minor mode for enabling navigating sources gpr_query."
+  :lighter nil :interactive nil)
+
 ;;;###autoload
 (define-derived-mode ada-mode prog-mode "Ada"
   "The major mode for editing Ada code."
@@ -1685,7 +1673,7 @@ Unless WAIT, does not wait for parser to respond. Returns the parser object."
   ;; elsewhere before Ada files opened; start eglot in
   ;; wisi-select-prj.
   ;;
-  ;; In order to accomodate 1 and 2, we call ada-eglot-setup and wisi-setup in
+  ;; In order to accommodate 1 and 2, we call ada-eglot-setup and wisi-setup in
   ;; ada-mode-post-local-vars.
   (add-hook 'hack-local-variables-hook #'ada-mode-post-local-vars nil t)
   )
@@ -1722,16 +1710,20 @@ Unless WAIT, does not wait for parser to respond. Returns the parser object."
 
   (setq-local wisi-disable-parser nil)
 
-  (when (or (eq ada-xref-backend   'eglot)
-	    (eq ada-indent-backend 'eglot)
-	    (eq ada-face-backend   'eglot))
+  (when (or (eq ada-diagnostics-backend 'eglot)
+	    (eq ada-face-backend        'eglot)
+	    (eq ada-indent-backend      'eglot)
+	    ;; ada-statement-backend cannot be 'eglot
+	    (eq ada-xref-backend        'eglot))
     (require 'ada-eglot)
     (when (fboundp 'ada-eglot-setup)
       (ada-eglot-setup)))
 
-  (when (or (eq ada-xref-backend   'other)
-	    (eq ada-indent-backend 'other)
-	    (eq ada-face-backend   'other))
+  (when (or (eq ada-diagnostics-backend 'other)
+	    (eq ada-face-backend        'other)
+	    (eq ada-indent-backend      'other)
+	    (eq ada-statement-backend   'other)
+	    (eq ada-xref-backend        'other))
     (when ada-other-backend-setup-function
       (funcall ada-other-backend-setup-function)))
 
@@ -1748,6 +1740,12 @@ Unless WAIT, does not wait for parser to respond. Returns the parser object."
 		 ada-process-parse-exec))
 	(setq-local wisi-disable-parser t)))
 
+  (when (eq ada-statement-backend 'wisi)
+    (ada--statement-minor-mode))
+
+  (when (eq ada-xref-backend 'gpr_query)
+    (ada--gpr-query-minor-mode))
+
   ;; wisi-setup tolerates parser nil.
   (wisi-setup
    :indent-calculate '(ada-wisi-comment)
